;
;	ShellNotify.au3 - Erzeugt ein Symbol im Infobereich
;	                - der Taskleiste.
;                    - Copyright  2009, Greenhorn
;

#NoTrayIcon

#include <WindowsConstants.au3>
#include <GUIConstantsEx.au3>
#include <Constants.au3>

#include "Menu.au3"


Global $user32dll
Global $shell32dll
Global $kernel32dll


If (not $user32dll) Then _
	Global $user32dll = DllOpen ("user32.dll")

If (not $shell32dll) Then _
	Global $shell32dll = DllOpen ("shell32.dll")

If (not $kernel32dll) Then _
	Global $kernel32dll = DllOpen ("kernel32.dll")


Global Const $MK_LBUTTON          = 0x0001
Global Const $MK_RBUTTON          = 0x0002
Global Const $MK_SHIFT            = 0x0004
Global Const $MK_CONTROL          = 0x0008
Global Const $MK_MBUTTON          = 0x0010
Global Const $MK_XBUTTON1         = 0x0020
Global Const $MK_XBUTTON2         = 0x0040

Global Const $WM_RBUTTONDBLCLK	= 0x0206


; Shell notify icon constants
Global Const $NIM_ADD	= 0
Global Const $NIM_MODIFY	= 1
Global Const $NIM_DELETE	= 2

Global Const $NOTIFYICON_VERSION	= 3
Global Const $NOTIFYICON_VERSION_4	= 4

Global Const $NIM_SETFOCUS	= 3
Global Const $NIM_SETVERSION	= 4

Global Const $NIN_POPUPOPEN	= $WM_USER+6
Global Const $NIN_POPUPCLOSE	= $WM_USER+7

Global Const $NIF_MESSAGE	= 0x00000001
Global Const $NIF_ICON		= 0x00000002
Global Const $NIF_TIP		= 0x00000004
Global Const $NIF_STATE		= 0x00000008
Global Const $NIF_INFO		= 0x00000010
Global Const $NIF_GUID		= 0x00000020

Global Const $NIIF_NONE		= 0x00000000
Global Const $NIIF_INFO		= 0x00000001
Global Const $NIIF_WARNING	= 0x00000002
Global Const $NIIF_ERROR		= 0x00000003
Global Const $NIIF_ICON_MASK	= 0x0000000F
Global Const $NIIF_NOSOUND	= 0x00000010

Global Const $NIS_HIDDEN		= 0x00000001
Global Const $NIS_SHAREDICON	= 0x00000002

;

; ShellNotify Nachricht
Global Const $WM_SHELLNOTIFY 	= $WM_USER + 5

Global Const $NIS_SHOW	= 1
Global Const $NIS_HIDE	= 2


; --- NOTIFYICONDATA
Global Const $NOTIFYICONDATA = _
	'dword cbSize;'& _
	'hwnd hWnd;'& _
	'uint uID;'& _
	'uint uFlags;'& _
	'uint uCallbackMessage;'& _
	'ptr hIcon;'& _
	'wchar szTip [128];'& _
	'dword dwState;'& _
	'dword dwStateMask;'& _
	'wchar szInfo [256];'& _
	'uint uTimeout;'& _ ;'union { uint uTimeout; uint uVersion; }
	'wchar szInfoTitle [64];'& _
	'dword dwInfoFlags;'& _
	'wchar guidItem;'& _
	'ptr   hBalloonIcon;'



; Interne Struktur fr Tray-Symbol
Global Const $SHELLNOTIFYDATA = _
	"uint  uIconID;"& _
	"ptr   hIcon;"& _
	"ptr   hMenu;"& _
	"uint  uMenuID;"& _
	"wchar szTip [128];"& _
	"hwnd  hWnd;"& _
	"int   uNotifyMsg;"

; Struktur fr Tray-Symbol-Informationen
Global Const $NOTIFYICONINFO = _
	"uint  uIconID;"& _
	"ptr   hIcon;"& _
	"ptr   hMenu;"& _
	"uint  uMenuID;"& _
	"wchar szTip [128];"& _
	"hwnd  hWnd;"

; Tray icon window
Global $g_hwndNotify = 0x00000000

; Tray icon data
Global $g_SND  [1] = [0]


;

OnAutoItExitRegister ("OnShellNotifyExit")

Func OnShellNotifyExit ( )

	For $i = 0 To UBound ($g_SND) - 1

		; TODO:
		; Ressourcen aus g_TRAYICONDATA freigeben !
		DllCall ($user32dll, "int", "DestroyIcon", _
						"ptr", DllStructGetData ($g_SND[$i], "hIcon"))
		$g_SND[$i] = 0

	Next

	DllClose ($user32dll)
	DllClose ($shell32dll)
	DllClose ($kernel32dll)


EndFunc

;

GUIRegisterMsg ($WM_SHELLNOTIFY,   "WM_SHELLNOTIFY")

Func WM_SHELLNOTIFY  ($hwnd, $message, $wParam, $lParam)

	Local $aPos
	Local $hmenu
	Local $bShowMenu
	Local $uNotifyMsg


	Switch (BitAND($lParam, 0xFFFF))

		Case $WM_LBUTTONDOWN
			$lParam = 1

		Case $WM_LBUTTONUP
			$lParam = 2

		Case $WM_LBUTTONDBLCLK
			$lParam = 4

		Case $WM_RBUTTONDOWN
			$lParam = 8

		Case $WM_RBUTTONUP
			$lParam = 16

		Case $WM_RBUTTONDBLCLK
			$lParam = 32

		Case Else
			Return 0

	EndSwitch

	$uNotifyMsg = DllStructGetData ($g_SND[$wParam], "uNotifyMsg")

	$bShowMenu = BitAND ($uNotifyMsg, $lParam)

	If ($bShowMenu or $uNotifyMsg == 32) Then

		$aPos = MouseGetPos ( )

		$hmenu = DllStructGetData ($g_SND[$wParam], "hMenu")

		$hwnd  = DllStructGetData ($g_SND[$wParam], "hWnd")

		SetForegroundWindow ($hwnd)

		TrackPopupMenuEx ($hmenu, $TPM_BOTTOMALIGN, $aPos[0], $aPos[1], $hwnd, 0)

		PostMessage ($hwnd, 0, 0, 0)

	EndIf

	Return 0

EndFunc

;

Func SetNotifyIconCommand ($uIconID, $uMsg)

	DllStructSetData ($g_SND[$uIconID], "uNotifyMsg", $uMsg)

EndFunc

Func SetNotifyIconState ($uIconID, $fState)

	Local $nid
	Local $lpnid


	If ($uIconID < 1) Then _
		Return 0

	If (not $fState) Then _
		Return 0

	$nid = DllStructCreate ($NOTIFYICONDATA)
	$lpnid = DllStructGetPtr ($nid)

	DllStructSetData ($nid, "cbSize",   DllStructGetSize ($nid))
	DllStructSetData ($nid, "hWnd",     DllStructGetData ($g_SND[$uIconID], "hWnd"))
	DllStructSetData ($nid, "uID",      DllStructGetData ($g_SND[$uIconID], "uIconID"))


	If ($fState == $NIS_SHOW) Then

		DllStructSetData ($nid, "uFlags",   BitOR ($NIF_ICON, $NIF_TIP, $NIF_MESSAGE))
		DllStructSetData ($nid, "uCallbackMessage", $WM_SHELLNOTIFY)
		DllStructSetData ($nid, "hIcon",    DllStructGetData ($g_SND[$uIconID], "hIcon"))
		DllStructSetData ($nid, "szTip",    DllStructGetData ($g_SND[$uIconID], "szTip"))

		Shell_NotifyIcon ($NIM_ADD, $lpnid)

		DllStructSetData ($nid, "uTimeout", $NOTIFYICON_VERSION)	; uVersion !!!

		Shell_NotifyIcon ($NIM_SETVERSION, $lpnid)

	ElseIf ($fState == $NIS_HIDE) Then

		Shell_NotifyIcon ($NIM_DELETE, $lpnid)

	EndIf


EndFunc

Func CreateNotifyPopupMenu ($uIconID)

	Local $hmenu
	Local $uMenuID


	If ($uIconID < 1) Then _
		Return 0


	$uMenuID = GUICtrlCreateContextMenu ( _
				GUICtrlCreateDummy ( ) _
				)

	$hmenu = GUICtrlGetHandle ($uMenuID)

	DllStructSetData ($g_SND[$uIconID], "hMenu",   $hmenu)
	DllStructSetData ($g_SND[$uIconID], "uMenuID", $uMenuID)

	Return $uMenuID

EndFunc

Func CreateNotifyIcon ($hwnd, $sToolTip = "", $sFileName = "", $nIconID = 0)

	Local $uSize
	Local $hIcon
	Local $ahIcon
	Local $cnIcons


	If (not $hwnd) Then

		If (not $g_hwndNotify) Then

			$g_hwndNotify = GUICreate ( _
							"", _
							1, 1, 9999, 9999, _
							BitOR ($WS_CAPTION, $WS_SYSMENU), _
							$WS_EX_TOOLWINDOW _
							)

			GUISetState (@SW_SHOW, $g_hwndNotify)
			ShowWindow ($g_hwndNotify, @SW_HIDE)

			$hwnd = $g_hwndNotify
		EndIf
	EndIf

	If (not $hwnd) Then _
		Return 0

	If (not $sToolTip) Then _
		$sToolTip = StringTrimRight (@ScriptName, 4)

	If (not $sFileName) Then _
		$sFileName = @AutoItExe

	$uSize = UBound ($g_SND)

	ReDim $g_SND[$uSize + 1]

	$ahIcon = DllStructCreate ("ptr [1]")

	$cnIcons = ExtractIconExW ($sFilename, $nIconID, 0, DllStructGetPtr ($ahIcon), 1)

	If (not $cnIcons) Then _
		Return 0

	$hIcon = DllStructGetData ($ahIcon, 1)

	$g_SND[$uSize] = DllStructCreate ($SHELLNOTIFYDATA)

	DllStructSetData ($g_SND[$uSize], "uIconID", $uSize)
	DllStructSetData ($g_SND[$uSize], "hWnd",    $hwnd)
	DllStructSetData ($g_SND[$uSize], "hIcon",   $hIcon)
	DllStructSetData ($g_SND[$uSize], "szTip",   $sToolTip)


	Return $uSize	; Tray icon ID !


EndFunc

Func DeleteNotifyIcon ($uIconID)

	Local $nid
	Local $lpnid
	Local $hIcon
	Local $uSize


	If ($uIconID < 1) Then _
		Return 0

	$nid = DllStructCreate ($NOTIFYICONDATA)
	$lpnid = DllStructGetPtr ($nid)

	DllStructSetData ($nid, "cbSize", DllStructGetSize ($nid))
	DllStructSetData ($nid, "uID",    $uIconID)
	DllStructSetData ($nid, "hWnd",   DllStructGetData ($g_SND[$uIconID], "hWnd"))

	Shell_NotifyIcon ($NIM_DELETE, $lpnid)


	Return 1

EndFunc

Func GetNotifyIconInfo ($uIconID, ByRef $stNOTIFYICONINFO)

	If ( ($uIconID <= 0) or _
		($uIconID >= UBound ($g_SND)) or _
		(not IsDllStruct ($stNOTIFYICONINFO))) Then _
		Return FALSE

	DllStructSetData ($stNOTIFYICONINFO, 1, DllStructGetData ($g_SND[$uIconID], 1))
	DllStructSetData ($stNOTIFYICONINFO, 2, DllStructGetData ($g_SND[$uIconID], 2))
	DllStructSetData ($stNOTIFYICONINFO, 3, DllStructGetData ($g_SND[$uIconID], 3))
	DllStructSetData ($stNOTIFYICONINFO, 4, DllStructGetData ($g_SND[$uIconID], 4))
	DllStructSetData ($stNOTIFYICONINFO, 5, DllStructGetData ($g_SND[$uIconID], 5))
	DllStructSetData ($stNOTIFYICONINFO, 6, DllStructGetData ($g_SND[$uIconID], 6))

	Return TRUE

EndFunc


;

Func ExtractIcon ($hInst, $lpszExeFileName, $nIconIndex)

	Local $aRes = DllCall ($shell32dll, 'ptr', 'ExtractIconW', _
								'ptr', $hInst, _
								'wstr', $lpszExeFileName, _
								'uint', $nIconIndex)
	Return $aRes[0]

EndFunc

Func ExtractIconExW ($sFile, $nIconIndex, $phiconLarge, $phiconSmall, $nIcons)

	Local $aRes = DllCall ($shell32dll, 'uint', 'ExtractIconExW', _
								'wstr', $sFile, _
								'int',  $nIconIndex, _
								'ptr',  $phiconLarge, _
								'ptr',  $phiconSmall, _
								'uint', $nIcons)
	Return $aRes[0]

EndFunc

Func GetWindowLong ($hWnd, $nIndex)

	Local $aRes = DllCall ($user32dll, 'long', 'GetWindowLongW', _
								'hwnd', $hWnd, _
								'int', $nIndex)
	Return $aRes[0]

EndFunc

Func PostMessage ($hWnd, $Msg, $wParam, $lParam)

	Local $aRes = DllCall ($user32dll, 'int', 'PostMessageW', _
								'hwnd', $hWnd, _
								'uint', $Msg, _
								'wparam', $wParam, _
								'lparam', $lParam)
	Return $aRes[0]

EndFunc

Func Shell_NotifyIcon ($dwMessage, $lpdata)

	Local $aRes = DllCall ($shell32dll, 'int', 'Shell_NotifyIconW', _
								'dword', $dwMessage, _
								'ptr', $lpdata)
	Return $aRes[0]

EndFunc

Func ShowWindow ($hWnd, $nCmdShow)

	Local $aRes = DllCall ($user32dll, 'int', 'ShowWindow', _
								'hwnd', $hWnd, _
								'int', $nCmdShow)
	Return $aRes[0]

EndFunc

Func SetForegroundWindow ($hWnd)

	Local $aRes = DllCall ($user32dll, 'int', 'SetForegroundWindow', _
								'hwnd', $hWnd)
	Return $aRes[0]

EndFunc

